function [E, t, C, info] = fparamin(a, b, cmin, cmax, opts, fun, varargin)
%function [E, t, C {,info}] = fparamin(a, b, cmin, cmax, opts, fun, varargin)
%
% find minima and multiplicities of vector-valued tension function
% fun(E) assuming they are lowest few (unsorted) singular values of a
% MPS problem. Note tension is assumed to be approximately linear in |E-Ej|.
%
% INPUTS:
% a,b      defines search interval, a<b
% cmin     min slope (eg Moler-Payne thm or pert analysis)
% cmax     max slope (should be within factor of a few from min slope)
% fun(varargin{:}) is the (vector-valued) tension function
%
% All options are optional (sensible defaults are assumed otherwise):
% opts.Cmin     min quadratic coeff for parabolic search to be happy
% opts.ttyp     typical tension to stop splitting at (start large, eg 1e-3)
% opts.recur    max recursion depth to go to in pre-parabolic splitting
% opts.maxparit max parabolic search iterations (starts afresh after a split)
% opts.maxdegen max degeneracy possible, ie max # func vals to output
% opts.select   if true, keep multiple mins found in each split_interval
% opts.v        verbosity: 0=silent, 1=text, 2=text+(graphics on current fig)
%
% OUTPUTS:
% E             list of ordinates of minima
% t             corresponding tension values
% C             corresp curvatures of squared tension function
% info.fevals   # func evals used
%
% barnett 8/10/06


verb = 0;
if isfield(opts, 'v')        % verbose
  verb = opts.v;
end
if ~isfield(opts, 'maxdegen')        % overall max degeneracy to cope with
  opts.maxdegen = 100;
end
overall_maxdegen = opts.maxdegen;


% TWO PHASES:

% (1) split into regions containing 0 or 1 eigenvalue, or degenerate ones...
[as,bs,ns,fcs,info] = split_interval(a, b, cmin, cmax, opts, fun,...
                                varargin{:});
info.as = as; info.bs = bs; info.ns = ns; % keep info
info.a = a; info.b = b;

if verb>1  % show the split intervals as staircase
  line([as';bs'], kron([1;1],9-(1:numel(as))/10));
end
  
% (2) use squared t(E), just its min value, to do splits and parabolic searches
if ~isfield(opts,'Cmin')
  opts.Cmin = cmin^2;      % convert to minimum curvature for func sq
end
f2 = @(E) fun(E,varargin{:}).^2;  % inline new function: squared f
E = []; t = []; C = []; old_bsi = NaN;
for i=1:numel(as)
  e = [as(i) (as(i)+bs(i))/2 bs(i)];   % set up triples for min_parabola
  if as(i)==old_bsi                   % we can reuse interval end func eval
    f = [f(3) fcs(i)^2 min(f2(bs(i)))];  % note fcs was unsquared
    info.fevals = info.fevals + 1;
  else                                % both endpoints need to be eval'ed
    f = [min(f2(as(i))) fcs(i)^2 min(f2(bs(i)))];
    info.fevals = info.fevals + 2;
  end
  opts.maxdegen = info.ns(i);         % use the degeneracy info from splt_intl
  [Ei,ti,Ci,infoi] = min_parabola(f2, e, f, opts.recur, opts);
  % now extract degeneracies for each entry in Ei...
  Eideg = [];  Cideg = [];
  timdeg = [];      % minimum tension, duplicated according to multiplicity
  for j=1:numel(Ei)
    ks = find(ti(:,j) < 2*opts.ttyp^2); %    ks = which t's below roughly ttyp?
    deg = numel(ks);    % how many of them?
    Eideg = [Eideg repmat(Ei(j), [1 deg])]; % duplicate ordinate deg times
    Cideg = [Cideg repmat(Ci(j), [1 deg])]; % (different C's cannot be known)
    timdeg = [timdeg sqrt(reshape(ti(ks,j), [1 deg]))];
  end
  E = [E Eideg];      % collect multiplicity-expanded data as a row vec
  C = [C Cideg]; t = [t timdeg];     % also single row vecs
  if verb
    disp(['           min_p got Ei = ' sprintf('%.12g ',Eideg) 'w/ ti = ' ...
          sprintf('%.3g ',timdeg) sprintf('\n')]);
  end
  info.fevals = info.fevals + infoi.fevals;
  old_bsi = bs(i);
end

% end

