function [x,f,C,info] = min_parabola(fun, e, f, recur, opts)
%function [x, f, C {,info}] = min_parabola(fun, e, f, recur, opts)
%
% use curvature>0 parabolic approx to find single min of positive function fun
% in an interval, or return empty if none found in interval.
%
% INPUTS:
% e          row vec of 3 ordinates in increasing order
% f          corresponding 3 function values
% fun        function (squared tension); if returns a vector, min is used
% recur      remaining possible recursive calls (max possible depth)
%
% Optional:
% opts.xtol gives stopping criteria when min located to accuracy on x-axis.
% opts.v = 1 gives text only, 2 gives plotting of sqrt(f) too.
% opts.Cmin
% opts.maxparit
% opts.maxdegen     gives max number of funcs higher vals to keep
%
% OUTPUTS:
% x          row vec of ordinates of minima
% f          corresp func vals (multiple rows if fun returned vectors)
% C          corresp curvatures of (squared) tension function at minima
% info.fevals counts # func evals used
%
% NOTE - some of the gather code in case of 2 minima found is untested!
%
% barnett 8/13/06

if ~isfield(opts, 'xtol')
  opts.xtol = 1e-4;          % default
end
verb = 0;
if isfield(opts, 'v')        % verbose
  verb = opts.v;
end
if ~isfield(opts, 'Cmin')
  opts.Cmin = 0;             % default min curvature
end
if ~isfield(opts, 'maxparit')
  opts.maxparit = 5;             % default max parabolic iterations
end
if ~isfield(opts, 'maxdegen')
  opts.maxdegen = 100;             % default max degeneracy possible
end

fevals = 0;
a = e(1); b = e(2); c = e(3); fa = f(1); fb = f(2); fc = f(3); % keep for later
if verb
  sprec = repmat('  ',[1 4-recur]);
  disp([sprec sprintf(...
      'min_p (rec=%d Cm=%g) e=%.8g,%.8g,%.8g f=%.3g,%.3g,%.3g', ...
      recur, opts.Cmin, e(1), e(2), e(3), f(1), f(2), f(3))]);
end

[A,B,C] = para_fit(e, f);  % start the para it cycle rolling...
if verb
  disp(sprintf('%sA,B,C = %g,%g,%g', sprec, A, B, C));
  if verb>1
    plot([e;e], [sqrt(f); 0 0 0], 'k+-');
  end
end
it = 1;
% ------------------------------------------------- try zero or more para its
while (C>opts.Cmin & it<opts.maxparit & (min(diff(e))>opts.xtol | it<=2))
  % note it<=2 here is to stop the original b being accidentally hit by B
  % and taken as evidence of having converged.
  B = B + (rand(1)-0.5)*opts.xtol; % jitter the minimum to avoid hitting abc
  [d, i] = sort(abs(e - B));  % find the 2 nearest to B, discard d dists
  e = [B e(i(1)) e(i(2))];
  f = [min(fun(B)) f(i(1)) f(i(2))];      % corresp func vals
  fevals = fevals + 1;
  [e, i] = sort(e); f = f(i);         % sort into growing along axis
  if verb
    disp(sprintf('%sdone para it #%d... abc=%.8g, %.8g, %.8g', ...
                 sprec, it, e(1),e(2),e(3)));
  end
  it = it+1;
  [A,B,C] = para_fit(e, f);
  if verb
    disp(sprintf('%sA,B,C = %g, %g, %g', sprec, A, B, C));
    if verb>1
      plot([e;e], [sqrt(f); 0 0 0], 'r*-');
      t = e(1)+(e(3)-e(1))*(0:.05:1);          % show best-fit parabola
      plot(t, sqrt(max(A + C*(t-B).^2,0)), 'm-');  % clip <0
    end
  end
end % ------------------------------------------------------------ end its

if C>=opts.Cmin
  if a<B & B<c       % inside and curv valid
    x = B;
    f = sort(fun(B));       % is this final func eval needed?
    fevals = fevals + 1;
    f = f(1:min(numel(f),opts.maxdegen));
    f = reshape(f, [numel(f) 1]); % save its higher vals as col vec  
    if verb>1
      plot(x, sqrt(min(f)), 'go');    
    end
  else               % outside but curv valid
    x = []; f = []; C = [];
    if verb
      disp([sprec sprintf('discarding %.8g, outside interval', B)]);
    end
  end
  
else                  % curv invalid
  if recur>0          % recursive depth not yet max'ed out..................
    if verb
      disp([sprec 'splitting...']);
    end
    s = (a+b)/2; % note: split the original interval, not result of para its
    [x1,f1,C1,i1] = min_parabola(fun, [a s b], [fa min(fun(s)) fb], ...
                                 recur-1, opts);
    s = (b+c)/2;
    [x2,f2,C2,i2] = min_parabola(fun, [b s c], [fb min(fun(s)) fc], ...
                                 recur-1, opts);
    fevals = fevals + 2 + i1.fevals + i2.fevals;  % since 2 fun(s)'s
    x = [x1 x2]; C = [C1 C2];
    if isempty(f1)
      f = f2;
    else
      if isempty(f2)
        f = f1;
      else             % both f1 and f2 are interesting
        h = max(size(f1,1),size(f2,1)); % height of array
        f = [[f1; NaN*ones(h-size(f1,1),size(f1,2))] ...
             [f2; NaN*ones(h-size(f2,1),size(f2,2))]];
      end
    end
    if verb
      disp([sprec sprintf('gathered... minima (rec=%d): x = ',recur) ...
            sprintf('%.8g ', x) 'with f = ' sprintf('%.3g ', min(f))]);
      if isempty(x1)
        disp([sprec '(x1 was empty)']);
      end
      if isempty(x2)
        disp([sprec '(x2 was empty)']);
      end
    end
    if isfield(opts, 'select') & numel(x)>1
      mf = min(f);
      [mf,i] = min(mf);                 % use the lowest if >1 found
      x = x(i); C = C(i); f = f(:,i);
      if verb
        disp(sprintf('%sfrom those selecting # %d', sprec, i));
      end
    end
  else               % nothing left to try.................................
    if verb
      disp([sprec sprintf('invalid curv but do not split since recur=%d', ...
                          recur)]);
    end
    x = []; f = []; C = [];      % return emptyhanded
  end
end

info.fevals = fevals;

%end
