function [as,bs,ns,fcs,info] = split_interval(a, b, c, d, opts, fun, varargin)
%function [as, bs, ns {,fcs {,info}}] = ...
%             split_interval(a, b, c, d, opts, fun, varargin)
%
% Return subintervals in [a,b] containing 0-1 number
% of eigenvalues, as determined by c (min) and d (max) slope of a singular
% value-type function fun(E, varargin-passed-from-split_interval).
% Calls itself recursively to split the interval, until f_min gets to
% opts.ttyp, the critical value of f (ie an upper bound on tension min).
% Resulting interval j is [as(j), bs(j)] containing ns(j) eigenvalues.
% as,bs,ns are column vectors. fcs is lowest f val at center of each interval
%
% info.fevals returns # func evals used
%
% barnett 8/10/06

if ~isfield(opts, 'ttyp')
  opts.ttyp = 1e-2;          % default typical tension to stop splitting at
end
verb = 0;
if isfield(opts, 'v')        % verbose
  verb = opts.v;
end

E = (b+a)/2;
D = (b-a)/2;
f = fun(E, varargin{:});     % assume f>=0
fevals = 1;                  % counts the above func eval

nmin = numel(find(f<c*D));   % compute possible numbers of eigvals in range
nmax = numel(find(f<d*D));
fcs = min(f);
if verb
  if verb>1
    hold on; plot(E, f, 'b.'); line([E;E], [c*D;d*D]);
  end
  disp(sprintf(...
      'splt_intl: [%15.10g,%15.10g]   cenminf2=%7.2g,    has: [%d,%d]',...
               a,b, fcs, nmin, nmax));
end

if nmax==0
  as = []; bs = []; ns = []; fcs = [];
elseif (nmax==1 | c*D<opts.ttyp)                 % ok, stop
  as = a; bs = b; ns = nmax;
else                             % not definite: binary subdivide
  [as1, bs1, ns1, fcs1, info1] = split_interval(a, E, c, d, opts, fun,...
                                               varargin{:});
  % only call again if not enough info available...
  [as2, bs2, ns2, fcs2, info2] = split_interval(E, b, c, d, opts, fun,...
                                               varargin{:});
  as = [as1;as2];   bs = [bs1;bs2];             % concatenate results
  ns = [ns1;ns2];   fcs = [fcs1;fcs2];
  fevals = fevals + info1.fevals + info2.fevals;
end

% pass back info block
info.fevals = fevals;

%end

