function [q] = quad_polygon(b, M, type, opts)
%function [q] = quad_polygon(b, M, type {, opts})
%
% Generate quadrature struct for a polygon, given billiard object
% INPUTS:
% b              billiard object
% M              total # bdry pts desired
% type           't' trapezoid, 'c' chebychev
% opts.v         0,1,2 verbosity
% OUTPUTS:
% q              struct containing quadrature info on bdry and interior
%
% barnett 8/12/06

verb = 0;
if nargin>3
  if isfield(opts, 'v')        % verbose
    verb = opts.v;
  end
end
q = [];
q.M = ceil(M);

% boundary points
nc1 = circshift(b.c1, [0 -1]); nc2 = circshift(b.c2, [0 -1]);  % next corner
q.m = zeros(1,b.nc);      % # bdry pts on each edge
q.x1 = []; q.x2 = []; q.nx1 = []; q.nx2 = []; q.y1 = []; q.y2 = []; q.w = [];
for i=1:b.nc
  if i<b.nc
    q.m(i) = floor(q.M*b.l(i)/b.perim + 0.5);
  else
    q.m(i) = q.M - sum(q.m(1:b.nc-1));              % arrange total to be M
  end
  r = ((1:q.m(i))' - 0.5)/q.m(i);
  if type=='t'
    f = r;
    q.w = [q.w; (b.l(i)/q.m(i)) * ones(size(r))]; % trapezoid (uniform) weights
  elseif type=='c'
    f = (1-cos(pi*r))/2;                     % cheby pts
    q.w = [q.w; (b.l(i)/q.m(i)) * ones(size(r))]; % uniform weights for now
  else
    disp(sprintf('quad_polygon: invalid type'));
  end
  q.x1 = [q.x1; nc1(i)*f+b.c1(i)*(1-f)]; q.x2 = [q.x2; nc2(i)*f+b.c2(i)*(1-f)];
  q.nx1 = [q.nx1; sin(b.th(i))*ones(size(r))];  % outward normals
  q.nx2 = [q.nx2; -cos(b.th(i))*ones(size(r))];
end

% now create uniform lattice of points clipped to interior
if nargin>3 & isfield(opts,'dy')
  dy = opts.dy;
else
  dy = 0.08;             % default grid spacing
end
[y1, y2] = meshgrid(b.bbox(1):dy:b.bbox(2), b.bbox(3):dy:b.bbox(4));
inside = inside_bil(b, y1, y2);
q.y1 = y1(inside); q.y2 = y2(inside);
q.Mi = numel(q.y1);                                % determine # interior pts
q.wi = repmat(b.area/q.Mi, size(q.y1));            % interior weights

if verb>1   % graphical quadrature geometry feedback
  figure; hold on; show_quad(q);
  title('quad: bdry surf normals, interior pts');axis equal;
end

%end
