% Compute triangle neumann eigenmodes via MPS with corner-adapted basis.
% Alex Barnett, for Peter Doyle project,  7/31/06, general polygon 8/13/06

% The triangle has vertices (0,0), (1,0) and (xc,yc) with yc>0.
% The corresponding angles (in radians) are alpha, beta and gamma,
% What you specify are alpha, beta. Therefore the area depends on these.

alpha = pi/5;                          % choose angles
beta = 3*pi/5;

n = 10;                    % choose rough number desired levels
verb = 1;                  % verbosity: 0=silent, 1=text, 2=text+graphics
eta = 1.8;                 % basis convergence param, typ 1.8
btype = 'b';               % basis: 'b' both, 'n' neumann, 'on' one-corner
ppw = 20;                  % boundary pts per wavelength
bcs = []; bcs.type = 'd';  % choose BCs, type = 'n' or 'd' etc
meth = []; meth.nor = 'b'; % choose method, 'i' interior or 'b' bdry norm
meth.gep = 'v';            % method for gen eig prob.
mopt = []; mopt.ttyp=1e-5; % changed from 1e-4 which miscounted close ones
mopt.recur=3;    % minimization search options
mopt.maxparit=5; mopt.select=0; mopt.v=verb; mopt.maxdegen=100; mopt.xtol=1e-2;

% set up triangle corner locations given the 2 angles
if alpha+beta>=pi | alpha<0 | beta<0
  disp('invalid triangle angles!');
end
s = cot(alpha); t = cot(beta);
yc = 1/(s+t); xc = yc*s;
c1 = [0 1 xc]; c2 = [0 0 yc];
bil = bil_polygon(c1, c2);    % create a polygon billiard object

Emin = (pi/max(bil.l))^2;     % small enough to catch lowest Neu or Dir mode
Emax = (4*pi/bil.area)*n;     % use first Weyl term to estimate where to stop
width = 100;                  % adjust? dep on area
E = []; t = []; C = []; info = [];    % keep info about each search window
if verb>1, figure; end
tic;
% main loop over windows in 1 to Emax ---------------------------------
for a = Emin:width:Emax
  b = min(a+width,Emax);      % current window is [a,b]
  quad = quad_polygon(bil, ppw*bil.perim*sqrt(b)/(2*pi), 't'); % quadrature
  bas = setup_corner_basis(bil, eta*sqrt(b), btype);
  if bcs.type=='d', cmin=0.09/sqrt(b); cmax=0.17/sqrt(b);
  else, cmin = .04; cmax = .11;
  end % depend on billiard shape/size (0.04,0.11 neu)
  if verb>0
    disp(sprintf('WINDOW [%g,%g]: c=[%g,%g] N=%d M=%d\n\n',...
                 a,b,cmin,cmax,bas.N,quad.M));
  end
  [Ew,tw,Cw,infow] = fparamin(a, b, cmin, cmax, mopt, @param_spec, ...
                          bcs, meth, bas, quad);
  drawnow;
  E = [E Ew]; t = [t tw]; C = [C Cw]; info = [info infow];
  save tridir.2_.6.mat
end % ------------------------------------------------------------
toc
%save tri.2_.6_ne=610.mat

figure; plot(E,t, '+'); title('boundary error norms'); xlabel('E');
% check against Weyl's law
if bcs.type=='n'
  figure; k_weyl = weyl(0, sqrt(E), -bil.perim, bil.area);
else
  figure; k_weyl = weyl(0, sqrt(E), bil.perim, bil.area);
end

% sort
[E,i] = sort(E); t = t(i); C = C(i);

% check tendency of curvatures
figure;loglog(E,sqrt(C),'+');
if bcs.type=='n', hline(cmin); hline(cmax);
else
  if meth.nor=='i' % compare against slope limits for bcs=d, nor=i...
    hold on; loglog(E, 0.25./sqrt(E), ':'); loglog(E, 0.5./sqrt(E), ':');
  else
   hold on; loglog(E, 0.09./sqrt(E), ':'); loglog(E, 0.17./sqrt(E), ':');
  end
end

%save tri.2_.6_ne=311.mat

%save tri.4_.25_ne=36_ppw20.mat



% 8/23/06--------------------------------------------------------------------
% check duplicates found are same as due to large tensions:
sqrt(E(find(diff(E)==0)))              % k values of supposed double eigvals
sqrt(E(find(t>1e-5)))                  % k values of bad tension ones
% looks like the doubles are all due to a low 2nd singular val being
% misidentified as a double. (Even though we're supposed to be limited
% by ns value. So ns must have been > 1 each time.
% Maybe need to change the tmin in accordance with E, since curvatures drop
% in the dirichlet case (bcs.type=d, meth.nor=b).

% 5 potential duplicates, but only 3 actual ones by Weyl..................
%j=53;k=1; %j = 52; k=2;
mopt.ttyp=1e-6; mopt.v = 2; figure;
a=7120; b = 7122; %a=info(j).as(k); b = info(j).bs(k);
quad = quad_polygon(bil, ppw*bil.perim*sqrt(b)/(2*pi), 't'); % quadrature
bas = setup_corner_basis(bil, eta*sqrt(b), btype);
cmin=0.09/sqrt(b); cmax=0.17/sqrt(b);
[Ew,tw,Cw,infow] = fparamin(a, b, cmin, cmax, mopt, @param_spec, ...
                          bcs, meth, bas, quad);
% set the bad E's to zero, resort, remove:
i = find(E>0);
E = E(i); t = t(i); C= C(i);

f = fopen('tridir.2_.6_ne=591_E.dat', 'w'); fprintf(f, '%.4f\n', E);fclose(f);


figure; plot([E;E], repmat([0;1], size(E)), 'b-'); hold on;
EE = E(find(diff(E)==0)); plot([EE;EE], repmat([0;1], size(EE)), 'r-');




% compute Doyle's sum, windowed, and checking convergence.
ss = 0:0.01:10;
ns = numel(ss);
figure; ne = numel(E);
%for ne = [100 numel(E)] 	
  k = repmat(sqrt(E(1:ne)'), [1 ns]); s = repmat(ss, [ne 1]);
  win = repmat(1-(E(1:ne)'/E(ne)).^2, [1 ns]);
  TrIs = 2*pi*ss .* (sum(win .* besselj(1,k.*s)./k, 1) + 1*ss/2);
  TrIsp = 2*pi*ss .* (sum(win .* besselj(0,k.*s), 1) + 1);
  plot(ss, TrIsp, 'b-', 'markersize', 1); hold on;
%end
xlabel('s'); ylabel('Tr(I_s)');title('windowed Doyle sum incl trivial eigenvalue');
legend('n_e=100', 'n_e=311');

print -depsc2 TrIs.4_.25_ne=217_triv.eps

% compute sweeps ----------------------------------------------------
figure; Es = 160:0.4:200;
ss = NaN*ones(100,numel(Es)); ppw = 30;
quad = quad_polygon(bil, ppw*bil.perim*sqrt(max(Es))/(2*pi), 't'); % quadrature
bas = setup_corner_basis(bil, eta*sqrt(max(Es)), btype);
for j=1:numel(Es)
  s = param_spec(Es(j), bcs, meth, bas, quad);
  ss(1:numel(s),j) = s;
end
plot(Es, ss, 'm.', 'markersize', 1); axis([min(Es) max(Es) 0 1]);


% end

